/**
 * 本函数实现了自动将黑白图片上色的功能。
 * 本函数依赖 bos 触发器，图片文件上传到 bos bucket 后，bos 触发本函数。
 * 函数首先从 bos 读取该图片，对图片 base64 编码后通过 aip sdk 发送给百度云-图像效果增强服务，再把处理后的图片上传到 bos 的目标 bucket。
 * 图像效果增强服务有多种图像处理能力，文档参考: https://cloud.baidu.com/doc/IMAGEPROCESS/s/nk3bcloer
 */

var AipImageProcess = require('baidu-aip-sdk').imageProcess;
const {BosClient} = require('@baiducloud/sdk');

// 从环境变量读取 APPID/AK/SK
var APP_ID = process.env.APP_ID;
var AIP_KEY = process.env.AIP_KEY;
var AIP_SECRET_KEY = process.env.AIP_SECRET_KEY;
var DEST_BUCKET = process.env.DEST_BUCKET;

// 声明访问 aip 和 bos 的 client
var aipClient = new AipImageProcess(APP_ID, AIP_KEY, AIP_SECRET_KEY);
var bosClient = undefined;

exports.handler = async (event, context) => {
    for (var i = 0; i < event.events.length; i++) {
        try {
            var record = event.events[i];
            var bucket = record.content.bucket;
            var objectKey = record.content.object;
            initBosClient(record.content);

            var response = await bosClient.getObject(bucket, objectKey);
            var buffer = response.body;

            var result = await aipClient.colourize(buffer.toString('base64'));
            if (result.image === undefined) {
                console.error(result);
                continue;
            }

            var buff = Buffer.from(result.image, 'base64');
            await bosClient.putObject(DEST_BUCKET, objectKey, buff);
        }
        catch (e) {
            console.error(e);
        }
    }
    return 'done';
};

function initBosClient(content) {
    if (bosClient) {
        return;
    }

    if (content.domain.startsWith('http') === false) {
        content.domain = 'http://' + content.domain;
    }

    var bosConfig = {
        credentials: {
            ak: process.env.BCE_ACCESS_KEY_ID,
            sk: process.env.BCE_ACCESS_KEY_SECRET,
        },
        sessionToken: process.env.BCE_SESSION_TOKEN,
        endpoint: content.domain
    };
    bosClient = new BosClient(bosConfig);
}
